"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const quidproquo_webserver_1 = require("quidproquo-webserver");
const awsLambdaUtils_1 = require("../../../awsLambdaUtils");
const isAuthValid_1 = require("./utils/isAuthValid");
const transformHttpEventHeadersToAPIGatewayProxyResultHeaders = (headers) => {
    return Object.keys(headers)
        .filter((header) => !!headers[header])
        .reduce((acc, header) => (Object.assign(Object.assign({}, acc), { [header]: headers[header] })), {});
};
const getProcessTransformEventParams = (serviceName) => {
    return ({ eventParams: [apiGatewayEvent, context] }) => __awaiter(void 0, void 0, void 0, function* () {
        const path = (apiGatewayEvent.path || '').replace(new RegExp(`^(\/${serviceName})/`), '/');
        const transformedEventParams = {
            path,
            query: Object.assign(Object.assign({}, (apiGatewayEvent.multiValueQueryStringParameters || {})), (apiGatewayEvent.queryStringParameters || {})),
            body: apiGatewayEvent.body,
            headers: apiGatewayEvent.headers,
            method: apiGatewayEvent.httpMethod,
            correlation: context.awsRequestId,
            sourceIp: apiGatewayEvent.requestContext.identity.sourceIp,
            isBase64Encoded: apiGatewayEvent.isBase64Encoded,
        };
        console.log(JSON.stringify(transformedEventParams, null, 2));
        return (0, quidproquo_core_1.actionResult)(transformedEventParams);
    });
};
const getProcessTransformResponseResult = (qpqConfig) => {
    // We might need to JSON.stringify the body.
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const headers = Object.assign(Object.assign({}, quidproquo_webserver_1.qpqWebServerUtils.getCorsHeaders(qpqConfig, {}, payload.transformedEventParams.headers)), (((_a = payload === null || payload === void 0 ? void 0 : payload.response) === null || _a === void 0 ? void 0 : _a.headers) || {}));
        return (0, quidproquo_core_1.actionResult)({
            statusCode: payload.response.status,
            body: payload.response.body || '',
            isBase64Encoded: payload.response.isBase64Encoded,
            headers: transformHttpEventHeadersToAPIGatewayProxyResultHeaders(headers),
        });
    });
};
const getProcessAutoRespond = (qpqConfig) => {
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        if (payload.transformedEventParams.method === 'OPTIONS') {
            return (0, quidproquo_core_1.actionResult)({
                status: 200,
                isBase64Encoded: false,
                body: '',
                headers: quidproquo_webserver_1.qpqWebServerUtils.getCorsHeaders(qpqConfig, payload.matchResult.config || {}, payload.transformedEventParams.headers),
            });
        }
        const authValid = yield (0, isAuthValid_1.isAuthValid)(qpqConfig, quidproquo_webserver_1.qpqWebServerUtils.getHeaderValue('Authorization', payload.transformedEventParams.headers), quidproquo_webserver_1.qpqWebServerUtils.getHeaderValue('x-api-key', payload.transformedEventParams.headers), (_a = payload.matchResult.config) === null || _a === void 0 ? void 0 : _a.routeAuthSettings);
        if (!authValid) {
            return (0, quidproquo_core_1.actionResult)({
                status: 401,
                isBase64Encoded: false,
                body: JSON.stringify({
                    message: 'You are unauthorized to access this resource',
                }),
                headers: quidproquo_webserver_1.qpqWebServerUtils.getCorsHeaders(qpqConfig, payload.matchResult.config || {}, payload.transformedEventParams.headers),
            });
        }
        return (0, quidproquo_core_1.actionResult)(null);
    });
};
const getProcessMatchStory = (qpqConfig) => {
    const routes = quidproquo_webserver_1.qpqWebServerUtils.getAllRoutes(qpqConfig);
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // Sort the routes by string length
        // Note: We may need to filter variable routes out {} as the variables are length independent
        const sortedRoutes = routes
            .filter((r) => r.method === payload.transformedEventParams.method ||
            payload.transformedEventParams.method === 'OPTIONS')
            .sort((a, b) => {
            if (a.path.length < b.path.length)
                return -1;
            if (a.path.length > b.path.length)
                return 1;
            return 0;
        });
        // Find the most relevant match
        const matchedRoute = sortedRoutes
            .map((r) => ({
            match: (0, awsLambdaUtils_1.matchUrl)(r.path, payload.transformedEventParams.path),
            route: r,
        }))
            .find((m) => m.match.didMatch);
        if (!matchedRoute) {
            return (0, quidproquo_core_1.actionResultError)(quidproquo_core_1.ErrorTypeEnum.NotFound, `route not found [${payload.transformedEventParams.path}] - [${quidproquo_webserver_1.qpqWebServerUtils.getHeaderValue('user-agent', payload.transformedEventParams.headers)}]`);
        }
        return (0, quidproquo_core_1.actionResult)({
            src: matchedRoute.route.src,
            runtime: matchedRoute.route.runtime,
            runtimeOptions: matchedRoute.match.params || {},
            // TODO: Make this aware of the API that we are eventing
            config: quidproquo_webserver_1.qpqWebServerUtils.mergeAllRouteOptions('api', matchedRoute.route, qpqConfig),
        });
    });
};
exports.default = (qpqConfig) => {
    // TODO: Make this aware of the API that we are eventing
    const serviceName = quidproquo_core_1.qpqCoreUtils.getApplicationModuleName(qpqConfig);
    return {
        [quidproquo_core_1.EventActionType.TransformEventParams]: getProcessTransformEventParams(serviceName),
        [quidproquo_core_1.EventActionType.TransformResponseResult]: getProcessTransformResponseResult(qpqConfig),
        [quidproquo_core_1.EventActionType.AutoRespond]: getProcessAutoRespond(qpqConfig),
        [quidproquo_core_1.EventActionType.MatchStory]: getProcessMatchStory(qpqConfig),
    };
};
