"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAuthValid = void 0;
const quidproquo_core_1 = require("quidproquo-core");
const verifyJwt_1 = require("../../../../logic/cognito/verifyJwt");
const getExportedValue_1 = require("../../../../logic/cloudformation/getExportedValue");
const getApiKeys_1 = require("../../../../logic/apiGateway/getApiKeys");
const awsNamingUtils_1 = require("../../../../awsNamingUtils");
const isAuthValidForCognito = (qpqConfig, authSettings, authHeader) => __awaiter(void 0, void 0, void 0, function* () {
    // If there are no auth settings ~ Its valid.
    if (!authSettings.userDirectoryName) {
        return true;
    }
    // We need a header to be able to auth
    if (!authHeader) {
        return false;
    }
    // Make sure we have a Bearer token
    const [authType, accessToken] = authHeader.split(' ');
    if (authType !== 'Bearer' || !accessToken) {
        return false;
    }
    // Grab the user directory to auth against
    const userDirectoryName = authSettings.userDirectoryName;
    const region = quidproquo_core_1.qpqCoreUtils.getApplicationModuleDeployRegion(qpqConfig);
    // Resolve the user pool id
    const userPoolId = yield (0, getExportedValue_1.getExportedValue)((0, awsNamingUtils_1.getCFExportNameUserPoolIdFromConfig)(userDirectoryName, qpqConfig, authSettings.serviceName, authSettings.applicationName), region);
    // Resolve the user pool client id
    const userPoolClientId = yield (0, getExportedValue_1.getExportedValue)((0, awsNamingUtils_1.getCFExportNameUserPoolClientIdFromConfig)(userDirectoryName, qpqConfig, authSettings.serviceName, authSettings.applicationName), region);
    // Verify the token
    return yield (0, verifyJwt_1.verifyJwt)(accessToken, userPoolId, userPoolClientId, 'access');
});
const isAuthValidForApiKeys = (qpqConfig, authSettings, apiKeyHeader) => __awaiter(void 0, void 0, void 0, function* () {
    const apiKeys = authSettings.apiKeys || [];
    if (apiKeys.length === 0) {
        return true;
    }
    const region = quidproquo_core_1.qpqCoreUtils.getApplicationModuleDeployRegion(qpqConfig);
    const application = quidproquo_core_1.qpqCoreUtils.getApplicationName(qpqConfig);
    const service = quidproquo_core_1.qpqCoreUtils.getApplicationModuleName(qpqConfig);
    const environment = quidproquo_core_1.qpqCoreUtils.getApplicationModuleEnvironment(qpqConfig);
    const feature = quidproquo_core_1.qpqCoreUtils.getApplicationModuleFeature(qpqConfig);
    const realApiKeys = yield (0, getApiKeys_1.getApiKeys)(region, ...apiKeys.map((apiKey) => {
        const apiKeyApplication = apiKey.applicationName || application;
        const apiKeyService = apiKey.serviceName || service;
        return (0, awsNamingUtils_1.getConfigRuntimeResourceName)(apiKey.name, apiKeyApplication, apiKeyService, environment, feature);
    }));
    const index = realApiKeys.findIndex((apiKey) => apiKey.value === apiKeyHeader);
    return index >= 0;
});
const isAuthValid = (qpqConfig, authHeader, apiKeyHeader, authSettings) => __awaiter(void 0, void 0, void 0, function* () {
    // If there are no auth settings ~ Its valid.
    if (!authSettings) {
        return true;
    }
    const cognitoValid = yield isAuthValidForCognito(qpqConfig, authSettings, authHeader);
    const authKeysValid = yield isAuthValidForApiKeys(qpqConfig, authSettings, apiKeyHeader);
    return cognitoValid && authKeysValid;
});
exports.isAuthValid = isAuthValid;
